@@ -1,6 +1,6 @@
 ##
 ##
-## Copyright (c) 2009-2012 Brandon Whitcher and Volker Schmid
+## Copyright (c) 2009-2014 Brandon Whitcher and Volker Schmid
 ## All rights reserved.
 ## 
 ## Redistribution and use in source and binary forms, with or without
@@ -50,23 +50,31 @@ readNIfTI <- function(fname, verbose=FALSE, warn=-1, reorient=TRUE,
   ## Warnings?
   oldwarn <- getOption("warn")
   options(warn=warn)
-
   if (verbose) {
     cat(paste("  fname =", fname), fill=TRUE)
   }
+  ## Separate path and file name for more robust extension stripping
+  pathVector <- unlist(strsplit(fname, "/"))
+  file.name <- pathVector[length(pathVector)]
+  path <- paste(pathVector[-length(pathVector)], collapse="/")
+  if (length(pathVector) > 1) {
+      fname <- paste(path, file.name, sep="/")
+  } else {
+      fname <- file.name
+  }
   ## Strip any extensions
   fname <- sub("\\.gz$", "", fname)
   fname <- sub("\\.nii$", "", fname)
   fname <- sub("\\.hdr$", "", fname)
   fname <- sub("\\.img$", "", fname)
-
+  ## Add all possible file extensions 
   nii <- paste(fname, "nii", sep=".")
   niigz <- paste(fname, "nii.gz", sep=".")
   hdr <- paste(fname, "hdr", sep=".")
   hdrgz <- paste(fname, "hdr.gz", sep=".")
   img <- paste(fname, "img", sep=".")
   imggz <- paste(fname, "img.gz", sep=".")
-
+  ## Check all possible file extensions
   if (file.exists(niigz)) {
     ## If compressed file exists, then upload!
     if (verbose) {
@@ -287,9 +295,6 @@ readNIfTI <- function(fname, verbose=FALSE, warn=-1, reorient=TRUE,
     warning(paste("scl_slope =", nim@"scl_slope", "and data must be rescaled."))
     data <- data * nim@"scl_slope" + nim@"scl_inter"
   }
-
-  ##
-  ##
   ##
   ## THE SLOW BIT FOLLOWS
   ##
@@ -309,6 +314,7 @@ readNIfTI <- function(fname, verbose=FALSE, warn=-1, reorient=TRUE,
   ## This is a right-handed coordinate system.  However, the exact
   ## direction these axes point with respect to the subject depends on
   ## qform_code (Method 2) and sform_code (Method 3).
+  ##
   if (reorient) {
     nim@.Data <- reorient(nim, data, verbose=verbose)
     nim@"reoriented" <- TRUE
@@ -341,6 +347,15 @@ readANALYZE <- function(fname, SPM=FALSE, verbose=FALSE, warn=-1) {
   if (verbose) {
     cat(paste("  fname =", fname), fill=TRUE)
   }
+  ## Separate path and file name for more robust extension stripping
+  pathVector <- unlist(strsplit(fname, "/"))
+  file.name <- pathVector[length(pathVector)]
+  path <- paste(pathVector[-length(pathVector)], collapse="/")
+  if (length(pathVector) > 1) {
+      fname <- paste(path, file.name, sep="/")
+  } else {
+      fname <- file.name
+  }
   ## Strip any extensions
   fname <- sub("\\.gz$", "", fname)
   fname <- sub("\\.hdr$", "", fname)
@@ -464,7 +479,7 @@ readANALYZE <- function(fname, SPM=FALSE, verbose=FALSE, warn=-1) {
   aim@"omin" <- readBin(fid, integer(), size=4, endian=endian)
   aim@"smax" <- readBin(fid, integer(), size=4, endian=endian)
   magic <- readBin(fid, raw(), n=4, endian=endian)
-  aim@"smin" <- as.integer(magic) # readBin(fid, integer(), size=4, endian=endian)
+  aim@"smin" <- readBin(magic, integer(), size=4, endian=endian)
   ## Test for "magic" field (should not exist)
   if (rawToChar(magic) == "ni1") { # now its actually NIfTI two-file format
     stop("This is in two-file NIfTI format, please use readNIfTI")
